#ifndef APP_HH_HEADER_GUARD
#define APP_HH_HEADER_GUARD

#include <iostream>      // std::cerr
#include <cstdlib>       // std::exit, EXIT_FAILURE
#include <cstring>       // std::memset
#include <cstdint>       // std::uint32_t, std::uint8_t
#include <unordered_map> // std::unordered_map
#include <string>        // std::string
#include <algorithm>     // std::max

#include <SDL2/SDL.h>

#include "math.hh"
#include "world.hh"

class App {
public:
	App(const char *title, int w, int h, float scale);
	~App();

	void run();

private:
	void render(double dt);
	void update(double dt);
	void handleEvent(SDL_Event &evt);

	SDL_Window    *win;
	SDL_Renderer  *ren;
	SDL_Texture   *tex;
	int            scrW, scrH, scrSize;
	std::uint32_t *pixels;
	SDL_Rect       viewport;

	const std::uint8_t *keyboard;

	int   winW, winH;
	float aspectRatio;
	bool  running;

	World world;

	// Screen x to ray direction map
	std::vector<double> xDirMap;
};

#endif
