#ifndef WORLD_HH_HEADER_GUARD
#define WORLD_HH_HEADER_GUARD

#include <vector>    // std::vector
#include <algorithm> // std::swap, std::find

#include "math.hh"

struct Tile {
	std::vector<const Linef*> lines;

	Tile();
};

struct RaycastResult {
	Vec2f hit;
	float shade;
};

struct Camera {
	Vec2f  pos;
	double dir, fov, radius;

	Camera(Vec2f pos, double dir, double fov, double radius);

	void moveForward (double scale);
	void moveBackward(double scale);
	void moveLeft    (double scale);
	void moveRight   (double scale);

	void turnLeft(double scale);
	void turnRight(double scale);

	void sanitizeDir();
};

class World {
public:
	int w, h;

	std::vector<std::vector<Tile>> tiles;
	std::vector<Linef> lines;

	Camera cam;

	World(int w, int h, const Camera &cam);

	const Tile &tileAt(int x, int y) const;

	RaycastResult raycast(double off) const;

	void addLines(const std::vector<Linef> &toAdd);

private:
	void findClosestLineIntersect(const Vec2i &p, const Linef &line,
	                              Vec2f &closestIntersect, const Linef *&closestLine) const;

	void initDDA(Vec2f start, Vec2f d, Vec2f &m, Vec2f &l, Vec2f &ul, Vec2i &p, Vec2i &step) const;

	void mapLineToTiles(const Linef &line);
	void mapLinesToTiles();
};

#endif
